import assert from 'assert';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

/**
 * Custom the assert tool
 */

function should(condition, message) {
  assert(condition, "[vuex-map-model]: ".concat(message));
}
/**
 * Get the raw type string of a value e.g. [object Object]
 */

var _toString = Object.prototype.toString;
/**
 * Strict object type check. Only returns true
 * for plain JavaScript objects.
 */

function isPlainObject(obj) {
  return _toString.call(obj) === '[object Object]';
}
function isArray(arr) {
  return _toString.call(arr) === '[object Array]';
}

function index (models) {
  should(isArray(models) || isPlainObject(models), 'only accepts array or object');

  var _models = normalizeModels(models);

  return Object.keys(_models).reduce(function (mappedModels, key) {
    var _models$key = _slicedToArray(_models[key], 3),
        fieldPath = _models$key[0],
        mutationType = _models$key[1],
        spread = _models$key[2];

    mappedModels[key] = Object.create(null);

    mappedModels[key].get = function () {
      return resolveField(this.$store.state, fieldPath);
    };

    mappedModels[key].set = function (val) {
      var payload = spread ? _defineProperty({}, key, val) : val;
      this.$store.commit(mutationType, payload);
    };

    return mappedModels;
  }, Object.create(null));
}

function getFieldName(path) {
  var fields = path.split('.');
  return fields[fields.length - 1];
}
/**
 * Location field by chain of properties in object
 */


function resolveField(state, path) {
  return path.split('.').reduce(function (r, key) {
    return r[key];
  }, state);
}

function normalizeModels(models) {
  if (isArray(models)) {
    return models.reduce(function (convertedModels, model) {
      var rawModel = normalizeModel(model);

      for (var index in rawModel) {
        var _rawModel$index = _slicedToArray(rawModel[index], 4),
            key = _rawModel$index[0],
            path = _rawModel$index[1],
            mutationType = _rawModel$index[2],
            spread = _rawModel$index[3];

        convertedModels[key] = [path, mutationType, spread];
      }

      return convertedModels;
    }, Object.create(null));
  }

  return models;
}
/**
 *
 * @param {Array} model
 * -- [fieldPath, mutaionType]
 * -- [fieldPath, [field1, field2, field3...], mutaionType]
 * -- [fieldPath, field1, field2..., mutaionType]
 * @returns [[key, fieldPath, mutationType]]
 */


function normalizeModel(model) {
  should(model.length > 1, 'the length of the model expression must greater than 1');
  should(isArray(model), 'model expression must be array');
  var fieldPath = model[0];
  var mutaionType = model[model.length - 1];
  var rootKey = getFieldName(fieldPath);

  if (model.length > 2) {
    var submodels = isArray(model[1]) ? model[1] : [model[1]];

    if (model.length > 3) {
      submodels = model.slice(1, -1);
    }

    return submodels.map(function (key) {
      return [key, "".concat(fieldPath, ".").concat(key), mutaionType, true];
    }).concat([[rootKey, fieldPath, mutaionType, false]]);
  }

  return [[rootKey].concat(_toConsumableArray(model))];
}

export default index;
